using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Terraria;
using TAPI;

namespace Bluemagic.Projectiles {
public class TrailHitbox : CustomHitbox
{
    public Stretch start;
    private Vector2 topLeft;
    private Vector2 bottomRight;

    public int duration
    {
        get
        {
            return (int)projectile.ai[0];
        }
        set
        {
            projectile.ai[0] = value;
        }
    }

    public int age
    {
        get
        {
            return (int)projectile.ai[1];
        }
        set
        {
            projectile.ai[1] = value;
        }
    }

    public void Add(Vector2 pos)
    {
        int expires = age + duration;
        Vector2 halfSize = projectile.Size / 2f;
        if(start == null)
        {
            start = new Stretch(pos, expires);
            topLeft = pos - halfSize;
            bottomRight = pos + halfSize;
        }
        else
        {
            start.Add(pos, expires);
            if(pos.X - halfSize.X < topLeft.X)
            {
                topLeft.X = pos.X - halfSize.X;
            }
            else if(pos.X + halfSize.X > bottomRight.X)
            {
                bottomRight.X = pos.X + halfSize.X;
            }
            if(pos.Y - halfSize.Y < topLeft.Y)
            {
                topLeft.Y = pos.Y - halfSize.Y;
            }
            else if(pos.Y + halfSize.Y > bottomRight.Y)
            {
                bottomRight.Y = pos.Y + halfSize.Y;
            }
        }
        projectile.Center = pos;
        PostAdd(pos);
    }

    public virtual void PostAdd(Vector2 pos) {}

    public bool Remove()
    {
        if(start.Remove())
        {
            start = start.next;
        }
        return start == null;
    }

    public override void AI()
    {
        age++;
        if(start == null)
        {
            Add(projectile.Center);
        }
        CustomBehavior();
        while(start.start.expires <= age)
        {
            if(Remove())
            {
                projectile.active = false;
                return;
            }
        }
        projectile.timeLeft = 2;
    }

    public virtual void CustomBehavior() {}

    public override bool Collides(CodableEntity entity)
    {
        return Collides(entity.position, entity.position + entity.Size);
    }

    public bool Collides(Vector2 corner1, Vector2 corner2)
    {
        if(start == null || corner2.X < topLeft.X || corner2.Y < topLeft.Y || corner1.X > bottomRight.X || corner1.Y > bottomRight.Y)
        {
            return false;
        }
        Stretch current = start;
        Vector2 halfSize = projectile.Size / 2f;
        while(current != null)
        {
            Vector2 stretchStart = current.start.position;
            Vector2 stretchEnd = current.end.position;
            if(!(corner2.X < stretchStart.X && corner2.X < stretchEnd.X) && !(corner1.X > stretchStart.X && corner1.X > stretchEnd.X) && !(corner2.Y < stretchStart.Y && corner2.Y < stretchEnd.Y) && !(corner1.Y > stretchStart.Y && corner1.Y > stretchEnd.Y))
            {
                Spot spot = current.start;
                while(spot != null)
                {
                    Vector2 pos = spot.position;
                    if(pos.X - halfSize.X <= corner2.X && pos.X + halfSize.X >= corner1.X && pos.Y - halfSize.Y <= corner2.Y && pos.Y + halfSize.Y >= corner1.Y)
                    {
                        return true;
                    }
                    spot = spot.next;
                }
            }
            current = current.next;
        }
        return false;
    }

    public override int AttackDirection(CodableEntity target)
    {
        return -target.direction;
    }

    public virtual float GetAlpha(float timeLeft)
    {
        return 1f;
    }

    public override bool PreDraw(SpriteBatch sb)
    {
        Stretch current = start;
        while(current != null)
        {
            Spot spot = current.start;
            while(spot != null)
            {
                float timeLeft = (float)(spot.expires - age) / (float)duration;
                Color color = Lighting.GetColor((int)(spot.position.X / 16f), (int)(spot.position.Y / 16f));
                sb.Draw(Main.projectileTexture[projectile.type], spot.position - Main.screenPosition, null, color * GetAlpha(timeLeft), 0f, projectile.Size / 2f, 1f, SpriteEffects.None, 0f);
                CreateDust(spot.position);
                spot = spot.next;
            }
            current = current.next;
        }
        return false;
    }

    public virtual void CreateDust(Vector2 pos) {}

    public class Stretch
    {
        public Spot start;
        public Spot end;
        public Stretch next;
        public Stretch(Vector2 pos, int exp)
        {
            start = new Spot(pos, exp);
            end = start;
        }

        public void Add(Vector2 pos, int exp)
        {
            Stretch current = this;
            while(current.next != null)
            {
                current = current.next;
            }
            Vector2 direction = current.end.position - current.start.position;
            Vector2 nextDir = pos - current.end.position;
            if(direction.X * nextDir.X < 0f || direction.Y * nextDir.Y < 0f)
            {
                current.next = new Stretch(pos, exp);
            }
            else
            {
                current.end = current.end.Add(pos, exp);
            }
        }

        public bool Remove()
        {
            start = start.next;
            return start == null;
        }
    }

    public class Spot
    {
        public Vector2 position;
        public Spot next;
        public int expires;
        public Spot(Vector2 pos, int exp)
        {
            position = pos;
            expires = exp;
        }

        public Spot Add(Vector2 pos, int exp)
        {
            Spot end = this;
            while(end.next != null)
            {
                end = end.next;
            }
            end.next = new Spot(pos, exp);
            return end.next;
        }
    }
}}